//------------------------------------------------------------------------------
// File: CD3DFilter.h
//
// Desc: Basic filter class using a D3D device which it renders movies with.
//
//------------------------------------------------------------------------------

#include <d3dx9.h>
#include <d3dx9shader.h>
#include <d3dx9tex.h>
#include <fstream.h>


#define VERTEX_SHADER_LOCATION LPCTSTR("C:\\common\\D3DFilter\\BasicVertexShader.txt")
#define PIXEL_SHADER_LOCATION LPCTSTR("C:\\common\\D3DFilter\\BasicPixelShader.txt")
#define LOGFILE_LOCATION "c:\\cd3dfilter_logfile.txt"

// This define is used for writing out the current file and line when logging
#define LOCATION "\n" << __FILE__ << " : Line "<< __LINE__ << "\n"
//#define TIME << queryPerformanceCounter() <<


class CD3DFilter : public CTransformFilter,
		  public ID3DFilter,
		  public ISpecifyPropertyPages
{

public:

    static CUnknown * WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

    // Reveals IContrast & ISpecifyPropertyPages
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);

    DECLARE_IUNKNOWN;

	// abstract TransformFilter methods
    HRESULT Transform(IMediaSample *pIn, IMediaSample *pOut);
    HRESULT CheckInputType(const CMediaType *mtIn);
    HRESULT CheckTransform(const CMediaType *mtIn,const CMediaType *mtOut);
    HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);
    HRESULT DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProperties);

    // ID3DFilter methods
    STDMETHODIMP setHWND(HWND *hwnd);
	STDMETHODIMP setParam1(float param);
	STDMETHODIMP setParam2(float param);
	STDMETHODIMP setParam3(float param);

    // ISpecifyPropertyPages method
    STDMETHODIMP GetPages(CAUUID *pPages);

	~CD3DFilter();

protected:

    // Constructor
    CD3DFilter(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr);

	// D3D help methods
	HRESULT InitD3D( HWND hWnd );
	HRESULT InitTexture();
	HRESULT RemoveTexture();
	HRESULT UploadTexture( IMediaSample * pSample );
	HRESULT DownloadTexture( IMediaSample * pSample );
	VOID Render();

	// D3D objects
	LPDIRECT3D9				m_pD3D;	
	LPDIRECT3DDEVICE9		m_pd3dDevice;			// d3d device
	LPDIRECT3DVERTEXBUFFER9 m_pVB;					// Buffer to hold vertices
	LPDIRECT3DTEXTURE9      m_pTexture;				// Our texture
	LPD3DXRENDERTOSURFACE	m_pRenderToSurface;		// Renderer
	LPDIRECT3DSURFACE9		m_pRenderTargetSurface; // The surface of the render target
	//LPDIRECT3DTEXTURE9		m_pRenderTargetSurface; // The surface of the render target
	WNDCLASSEX				m_wc;					// Needed for HWND
	HWND					hWnd;					// Hidden device window (for D3D)
	
	LPDIRECT3DVERTEXSHADER9 m_hBasicVertexShader;				// Vertex shader handle
	LPDIRECT3DPIXELSHADER9 m_hBasicPixelShader;				// Pixel shader handle

	// Shader constants
	float params[4];

	// Video properties
	LONG m_lVidWidth;	// Video width
	LONG m_lVidHeight;	// Video Height
	LONG m_lVidPitch;	// Video Pitch

	FLOAT frameTime; // Time between frames

	// Log file
	ofstream *m_logfile;

    // The number of buffers to request on the output allocator
    const long m_lBufferRequest;

    // Non interface locking critical section
    CCritSec m_D3DFilterLock;


}; // CD3DFilter

